/*
    File:            consol_awf1.cpp
    Author:          Tomas Krejci, 12/09/2008, revised 23/08/2012
    Purpose:         material model for saturated-nonsaturated one-phase flow (water) in deforming medium
    sources:         Lewis and Schrefler's book pp. 86-89, initial material parameters are set for benchmark on page n. 168;
    another sources: geomkp.pdf from michal Sejnoha;
                     
                     W. R. Gardner, Some steady-state solutions of the unsaturated moisture flow equation 
                     to evaporation from a water table, Soil Science 85 (1958), no. 4, 228-232;
                     
                     D. M. Potts and L. Zdravkovic, Finite element analysis in geotechnical engineering theory, 
		     Thomas Telford, London, 1999;

		     M. Th. van Genuchten, A closed equation for predicting the hydraulic conductivity of
		     unsaturated soils, Journal Soil Science Society of America 44 (1980), 892-898;
		     
		     P. Kuklik, J. Mares and M. Sejnoha (1999) The structural strenght of soil
		     from the isotropic consolidation point of view. Computational Mechanics for the Next
		     Millenium. Wang, Lee, Ang (ed.), Vol. 2, Elsevier, Singpore, 797-802;

		     P. Kuklik (2010) Preconsolidation, Structural Strength of soil, and its effect on subsoil 
		     upper structure intraction; Professor thesis

    unknowns:        number of unknowns=1, pw = liqud water pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "constrel.h"
#include "consol_awf1.h"
#include "globalt.h"

con_awf1mat::con_awf1mat()
{
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 2.167e6;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  phi0 = 0.297;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  //
  /////////////////////////////////////////////////////////
  ssat = 1.0; //
  //
  sirr = 0.33; //
  //
  alfa = 0.1; //
  //
  ksat = 0.1; //
  //
  gamaw = 10000.0; //
  /////////////////////////////////////////////////////////
  hp_min = 0.0; //
  htz = 0.14; //
  r = 100.0; //  
  /////////////////////////////////////////////////////////
  delta = 20.0; //
  expn = 2.0; //
  /////////////////////////////////////////////////////////
  k0 = 32e-11;//
  e0 = 0.52;//
  lam0 = 0.001;//
  lam = 0.03;//
  m = 9;//
  sigma_m_bar = 100000.0;//initial effective mean stress
  /////////////////////////////////////////////////////////
  kappa = 0.012;//
  sigma_m_eff_bar = 0.0;//initial (actual) effective mean stress = preconsolidation pressure
  sigma_m_tot_bar = 0.0;//initial (actual) total mean stress
  sigma_m_eff_init = 1.0e-3; //initial effective mean stress
  /////////////////////////////////////////////////////////

  mefel_units = 1.0;//basic units for pressures = Pa (Pascals)
}

con_awf1mat::~con_awf1mat()
{}


/**
   function computes conductivity %matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void con_awf1mat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err("unknown number of components of conductivity tensor is required",__FILE__,__LINE__,__func__);
  }
  }
}


/**
   function creates conductivity %matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);

  d[0][0] = kk;
}

/**
   function creates conductivity %matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);
  
  d[0][0] = kk;   d[0][1] = 0.0;
  d[1][0] = 0.0;  d[1][1] = kk;
}

/**
   function creates conductivity %matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void con_awf1mat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double kk;
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  kk = get_kww(pw,ipp);
  
  fillm(0.0,d);
  
  d[0][0]=kk;   d[0][1]=0.0;  d[0][2]=0.0;
  d[1][0]=0.0;  d[1][1]=kk;   d[1][2]=0.0;
  d[2][0]=0.0;  d[2][1]=0.0;  d[2][2]=kk;
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::matcap (double &cc,long ri,long ci,long ipp)
{
  double pw;
  
  pw = Tm->ip[ipp].av[0];
  cc = get_capww(pw,ipp);
}

/**
   function reads parameters
   
   @param in - input file

   12/9/2008, TKr
*/
void con_awf1mat::read(XFILE *in)
{
  
  xfscanf (in,"%k%m","waterflowtype",&waterflowtype_kwdset, &model_type);
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr);
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &mefel_units, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr);
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    xfscanf (in,"%lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &alfa, &ksat, &gamaw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &hp_min, &htz, &r, &ksat, &gamaw);
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &expn, &ksat, &delta, &gamaw);
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_bar, &sigma_m_tot_bar);
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from MEFEL cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_init);
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
}




/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  f = get_fw1(pw,ipp);
  fillm(0.0,d);
  d[0][0] = f*Tp->gr[0];
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  
  f = get_fw1(pw,ipp);
  
  fillm(0.0,d);
  d[0][0] = f*Tp->gr[0];
  d[1][0] = f*Tp->gr[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1mat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw;
  
  pw = Tm->ip[ipp].av[0];
  
  f = get_fw1(pw,ipp);
  
  fillm(0.0,d);
  d[0][0] = f*Tp->gr[0];
  d[1][0] = f*Tp->gr[1];
  d[2][0] = f*Tp->gr[2];
}




/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure

   @retval sw - degree of saturation

   12/9/2008, TKr
*/
double con_awf1mat::get_sw(double pw, long ipp)
{
  double sw,kr;
  double hp,theta,expm;
  double pc;
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    sw = 1.0 - (1.9722e-11)*pow(-pw,2.4279);//p. 167
    break;
  }
  case lewis_and_schrefler_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
    }
    else{
      //this below if for stability during tests
      pc = -pw;
      if(pc > 1)
	sw = 1.0 - 0.05125*log(pc);
      else
	sw = 1.0;
    }
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    kr = get_krw(pw,ipp);
    sw = sirr + (ssat - sirr)*kr;//Warick
    //sw = sirr + (ssat - sirr)*pow(kr,1.0/3.0);//alternative expression according to Fatt and Klikof
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    kr = get_krw(pw,ipp);
    sw = sirr + (ssat - sirr)*kr;//Warick
    //sw = sirr + (ssat - sirr)*pow(kr,1.0/3.0);//alternative expression according to Fatt and Klikof
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    hp = pw/gamaw;
    expm = 1.0 - 1.0/expn;
    theta = pow(1+(pow(delta*fabs(hp),expn)),-expm);
    sw = sirr + (ssat - sirr)*theta;
    break;
  }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    sw = 1.0;
    break;
  }
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    sw = 1.0;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(sw);
}


/**
   function computes partial derivative of degree of saturation with respect to pw, specific wter content
   @param pw - water pressure

   @retval dsw_dpw - partial derivative of degree of saturation with respect to pw

   12/9/2008, TKr
*/
double con_awf1mat::get_dsw_dpw(double pw, long ipp)
{
  double dsw_dpw;
  double dkr_dpw;
  double hp,expm;
  double pc;

  dsw_dpw = 0.0;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    dsw_dpw = 2.4279*(1.9722e-11)*pow(-pw,(2.4279-1.0));//p. 167
    break;
  }
  case lewis_and_schrefler_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      dsw_dpw = Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
      dsw_dpw = dsw_dpw/mefel_units;
    }
    else{
      //this below if for stability during tests
      pc = -pw;
      if(pc > 1)
	dsw_dpw = 0.05125/pc;
      else
	dsw_dpw = 0.0;
    }
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    dkr_dpw = exp(alfa*pw/gamaw)*alfa/gamaw;
    dsw_dpw = (ssat - sirr)*dkr_dpw;//Warick
    //dsw_dpw = (ssat - sirr)*alfa/3.0*pow(dkr_dpw,-2.0/3.0);//alternative expression according to Fatt and Klikof
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    hp = pw/gamaw;
    dkr_dpw = pow(10,(hp - hp_min)*log10(r)/htz)*log(10)*log10(r)/gamaw/htz;
    dsw_dpw = (ssat - sirr)*dkr_dpw;//Warick
    //dsw_dpw = (ssat - sirr)*alfa/3.0*pow(dkr_dpw,-2.0/3.0);//alternative expression according to Fatt and Klikof
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    hp = pw/gamaw;
    expm = 1.0 - 1.0/expn;
    dsw_dpw = (expm*expn*delta*(ssat - sirr)*pow(delta*fabs(hp),(expn-1)))/
      (pow(1+(pow(delta*fabs(hp),expn)),(expm+1)))/gamaw;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(dsw_dpw);
}


/**
   function computes water relative permeability
   @param pw - water pressure
   
   @retval krw - water relative permeability

   12/9/2008, TKr
*/
double con_awf1mat::get_krw(double pw, long ipp)
{
  double krw,s;
  double hp,expm;
  
  krw=1.0;
  
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    s = get_sw(pw,ipp);
    krw = 1.0+2207*pow((1.0-s),1.0121); //p. 167
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    //s = get_sw(pw,ipp);
    //this formula is teporarily here
    //krw = 1.0+2207*pow((1.0-s),1.0121); //p. 167
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    krw = exp(0.1*pw/gamaw);
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    hp = pw/gamaw;
    krw = pow(10,(hp - hp_min)*log10(r)/htz);
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    expm = 1.0 - 1.0/expn;
    hp = pw/gamaw;
    krw = pow(1.0-pow(delta*fabs(hp),expn)*pow(1.0+pow(delta*fabs(hp),expn),-expm),2.0)/
      pow(1.0+pow(delta*fabs(hp),expn),(expm/2.0));
    //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(3.0/expm)));//alternative expression according to Fatt and klikof
    //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(1.0/expm)));//alternative expression according Warrick
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(krw);
}


/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double con_awf1mat::give_porosity(long ipp)
{
  double por;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    por = phi0; //constant value
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book
    if(Tm->nontransq != NULL){
      por = Tm->givenontransq(porosity, ipp);// from mefel
    }
    else
      por = phi0;//for testing
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    por = phi0; //constant value
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    por = phi0; //constant value
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    por = phi0; //constant value
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(por);
}


/**
   function computes intrinsic permeability

   @retval kintr - intrinsic permeability

   12/9/2008, TKr
*/
double con_awf1mat::get_kintr()
{
  return(kintr);
}


/**
   function returns Biot's constant

   @retval alpha - Biot's constant

   12/9/2008, TKr
*/
double con_awf1mat::get_alpha()
{
  return(alpha);
}


/**
   function returns kw bulk modulus of water [Pa]

   @retval kw - bulk modulus of water [Pa]

   12/9/2008, TKr
*/
double con_awf1mat::get_kw()
{
  return(kw);
}


/**
   function returns muw water viscosity [Pa.s]

   @retval muw - water viscosity [Pa.s]

   12/9/2008, TKr
*/
double con_awf1mat::get_muw()
{
  return(muw0);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kww - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1mat::get_kww(double pw, long ipp)
{
  double s,ds;
  double krw,kww;
  double ks;
  double e,eps_v,sigma_m_eff,e0_bar;
  double a,b,c,e0_hat,lam_hat;


  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    krw = get_krw(pw,ipp);
    kww = krw*kintr/muw0;//p. 89
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    krw = get_krw(pw,ipp);
    kww = krw*kintr/muw0;//p. 89
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    ks  = ksat/86400.0;
    krw = get_krw(pw,ipp);
    kww = krw*ks/gamaw;
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    ks  = ksat/86400.0;
    krw = get_krw(pw,ipp);
    kww = krw*ks/gamaw;
    break;
  }
  case van_genuchten:{//partially saturated medium = Van Genuchten model
    ks  = ksat/86400.0;
    krw = get_krw(pw,ipp);
    kww = krw*ks/gamaw;    
    break;
  }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    
    sigma_m_eff = sigma_m_tot_bar + pw;

    /*
      if (sigma_m_eff_bar >= 0.0)
      e0_bar = e0;
      else
      e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);
      
      if(sigma_m_eff > sigma_m_eff_bar){//swelling (unloading) line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      //e = e0_bar*(1 + eps_v) + eps_v;
      e = -kappa*log(-sigma_m_eff) + e0_bar;//another expression
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
      }
      else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      //e = e0*(1 + eps_v) + eps_v;
      e = -lam*log(-sigma_m_eff) + e0;//another expression
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
      }
    */

    e0_bar = (kappa - lam)*log(-sigma_m_eff_bar) + e0;
    c = e0_bar;
    b = -kappa;
    a = -b*(lam/e0) - c*(lam/e0)*(lam/e0);
    e0_hat = -2*a*(log(-sigma_m_eff))+a*(log(-sigma_m_eff))*(log(-sigma_m_eff)) + b*(log(-sigma_m_eff)) + c;
    lam_hat = -2*a*(log(-sigma_m_eff));

    eps_v = -lam_hat/(1+e0_hat)*log(-sigma_m_eff);
    e = e0*(1 + eps_v) + eps_v;
    kww = -k0*(1+e0_hat)/lam_hat/gamaw*sigma_m_eff*pow((e/e0),m);
    
    break;
  }
  case kuklik_camclay_mech:{
    //isotropic consolidation according to Kuklik with changing porosity from cam-clay model; 
    //parameters are taken from MEFEL
    if(Tm->nontransq != NULL){
      //tady pokus jestli seje predavani pro Masina:??!!
      s = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
      ds = Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
      
      sigma_m_eff_bar = Tm->givenontransq(precons_press, ipp); //actual precosolidation pressure
      sigma_m_eff = Tm->givenontransq(mean_stress_eff, ipp); //actual effective mean stress
      if(sigma_m_eff >= sigma_m_eff_init)//this is for pore pressure (pressure is negative)
	sigma_m_eff = sigma_m_eff_init;//due to stability of computing
    }
    
    //porosity at zero pressure
    e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);
    if(sigma_m_eff >= sigma_m_eff_bar){//swelling line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      e = -kappa*log(-sigma_m_eff) + e0_bar;//another expression: e = e0_bar*(1 + eps_v) + eps_v;
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      e = -lam*log(-sigma_m_eff) + e0;//another expression: e = e0*(1 + eps_v) + eps_v;
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    
    /* // alternative computation - quadratic approximation aroung bifurcation point (NCL and swelling line)
       e0_bar = (kappa - lam)*log(-sigma_m_eff_bar) + e0;
       c = e0_bar;
       b = -kappa;
       a = -b*(lam/e0) - c*(lam/e0)*(lam/e0);
       e0_hat = -2*a*(log(-sigma_m_eff))+a*(log(-sigma_m_eff))*(log(-sigma_m_eff)) + b*(log(-sigma_m_eff)) + c;
       lam_hat = -2*a*(log(-sigma_m_eff));
       
       eps_v = -lam_hat/(1+e0_hat)*log(-sigma_m_eff);
       e = e0*(1 + eps_v) + eps_v;
       kww = -k0*(1+e0_hat)/lam_hat/gamaw*sigma_m_eff*pow((e/e0),m);
    */

    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(kww);
}



/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capww - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1mat::get_capww(double pw, long ipp)
{
  double sw,dsw_dpw,n,capww;  
    
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book  
    sw = get_sw(pw,ipp);
    dsw_dpw = get_dsw_dpw(pw,ipp);
    n = give_porosity(ipp);
    capww = (alpha-n)/ks*sw*(sw + dsw_dpw*pw)+n*sw/kw + n*dsw_dpw;//this equation is correct (different derivation)
    //capww = (alpha-n)/ks+n*sw/kw;//fully saturation state only for debug
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;  
    sw = get_sw(pw,ipp);
    dsw_dpw = get_dsw_dpw(pw,ipp);
    n = give_porosity(ipp);
    capww = (alpha-n)/ks*sw*(sw + dsw_dpw*pw)+n*sw/kw + n*dsw_dpw;//this equation is correct
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    dsw_dpw = get_dsw_dpw(pw,ipp);
    capww = phi0*dsw_dpw;
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    dsw_dpw = get_dsw_dpw(pw,ipp);
    capww = phi0*dsw_dpw;
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    dsw_dpw = get_dsw_dpw(pw,ipp);
    capww = phi0*dsw_dpw;
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    capww = 1.0; 
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    capww = 1.0; 
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(capww);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure

   @retval fw - first part for right-hand side for continutiy equation

   12/9/2008, TKr
*/
double con_awf1mat::get_fw1(double pw, long ipp)
{
  double krw,fw1;
  
  fw1 = 0.0;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book  
    krw = get_krw(pw,ipp);
    fw1 = krw*kintr/muw0*rhow;//p. 89
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;  
    krw = get_krw(pw,ipp);
    fw1 = krw*kintr/muw0*rhow;//p. 89
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    break;
  }
  case kuklik_camclay:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case kuklik_camclay_mech:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
    //from MEFEL cam-clay model
    krw = get_kww(pw,ipp);
    fw1 = krw*rhow;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }    

  return(fw1);

}




/**
   function computes all other variables at nodes
   @param compother - number of other components
   @param pw - water capillary pressure on actual node
   @param ipp - first integration point on element

   @retval other - other variable

   03/03/2011, TKr
*/

double con_awf1mat::get_othervalue(long compother,double pw, long ipp)
{
  double other;
  state_eq tt;

  switch (compother){
  case 0:{//capillary pressure
    other = -pw;
      break;
  }
  case 1:{//saturation
    other = get_sw(pw,ipp);
    break;
  }
  case 2:{//liquid water pressure
    other = pw;
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return (other);

}



/**
     function prints names of all other variables at nodes
     @param out - output rhle
     @param compother - number of other components

     03/03/2011, TKr
*/
void con_awf1mat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capillary pressure (Pa)");
    break;
  }
  case 1:{//saturation
    fprintf (out,"Degree of saturation ()       ");
    break;
  }
  case 2:{//liquid water pressure
    fprintf (out,"Pore water pressure (Pa)      ");
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function checks if computed unknowns are physically reasonable
   @param nv  - vector of unknowns
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void con_awf1mat::values_correction (vector &nv, long ipp)
{
  //  pore water pressure control
  water_pressure_check(nv[0],ipp);
}


/**
   function checks if water pressure is non-positive
   @param pw - pore water pressure
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void con_awf1mat::water_pressure_check(double &pw,long ipp)
{
  //this must be checked:
  //if (pw >= 0.0) 
  //pw = -1.0;
  
  //storing into gauss point
  //Tm->ip[ipp].av[0] = pw;
}

/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.

   24/08/2012, TKr
*/
void con_awf1mat::updateval (long ipp)
{
}


/**
   This function initializes material model data
   with respect of consistency parameter gamma.

   @param ipp - integration point number

   24/08/2012, TKr
*/
void con_awf1mat::initval(long ipp)
{
}



/**
  The function returns ordered dof names of primary unknowns 
  required by the model.

  @param dofname   - array of uknown name for particular nodal dofs (output)
                     dofname[i] = name of i-th nodal unknown (for names see aliast.h - enum namevart)
  @param ntm       - number of transported media = number of nodal dof = length of array dofname

  Created by Tomas Krejci according to Tomas Koudelka, 28/04/2014
*/
void con_awf1mat::give_dof_names(namevart *dofname, long ntm)
{
  if (ntm < 1)
  {
    print_err("the model defines %ld unknowns but number of transported media is %ld", 
              __FILE__, __LINE__, __func__, 1, ntm);
    abort();
  }
  dofname[0] = trf_water_press;
}




/**
   function returns water pressure

   @param ipp - integration point number

   @retval pw - pore water pressure
*/
double con_awf1mat::give_water_pressure(long ipp)
{
  double pw;

  pw = Tm->ip[ipp].av[0];

  return(pw);
}


/**
   function computes suction stress s = pw = -pc;
   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   25/9/2012, TKr
*/
double con_awf1mat::give_suction(long ipp)
{
  double pw,suction;
  
  pw = Tm->ip[ipp].av[0];

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    suction = pw;//this is correct, because capillary water pressure is negative
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    suction = pw;//this is correct, because capillary water pressure is negative

    suction = suction/mefel_units;//corrected units for mefel //basic units = Pa    
    break;
  }
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    suction = -pw;//tady promyslet??!!
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    suction = -pw;//tady promyslet??!!
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    suction = -pw;//tady promyslet??!!
    break;
  }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    suction = -pw;//tady promyslet??!!
    break;
  }
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    suction = -pw;//tady promyslet??!!
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(suction);
}


/**
   function computes suction stress (s = -pw = pc);
   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   25/9/2012, TKr
*/
double con_awf1mat::give_saturation_degree(long ipp)
{
  double pw,s;
  
  pw = Tm->ip[ipp].av[0];
  s = get_sw(pw,ipp);
  
  return(s);
}

/**
  The funtion marks required non-transport quantities in the array antq.

  @param antq - array with flags for used material types
                antq[i] = 1 => quantity type nontransquant(i+1) is required
                antq[i] = 0 => quantity type nontransquant(i+1) is not required

  @return The function does not return anything, but it may change content of antq array.
*/
void con_awf1mat::give_reqntq(long *antq)
{
  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    antq[porosity-1] = 1;
    break;
  }  
  case gardner_exponential:{//partially saturated medium = Exponential model, Gardner 
    break;
  }
  case potts_log_linear:{//partially saturated medium = Log linear model, Potts
    break;
  }
  case van_genuchten:{//partially saturated medium =Van Genuchten model
    break;
  }
  case kuklik_camclay:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case kuklik_camclay_mech:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    antq[precons_press-1] = 1;
    antq[mean_stress_eff-1] = 1;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    break;
  }
  default:{
    print_err("unknown model type is required", __FILE__, __LINE__, __func__);
  }
  } 
}
